.g-user-settings-container
  margin-top 10px
  border-radius 4px
  background-color #fafafa
  border 1px solid #e3e3e3
  padding 13px
  overflow auto

  .g-validation-failed-message
    margin-top 12px

#g-admin
  margin-left 10px

.g-api-keys-list-container
  margin-top 15px

  .g-api-keys-empty-message
    font-style italic

  .g-api-key-list-header
    font-weight bold
    font-size 16px
    color #666
    margin-bottom 5px

  .g-api-key-table
    width 100%

    .popover
      max-width  100%

    th
      padding 3px 4px 3px 3px
      background-color #723e92
      color white

    td
      padding 3px 2px 3px 3px

      button
        margin-right 4px

  .g-paginate-container
    float right

    .pagination
      margin 12px 0 0

  .g-api-key-list-bottom-buttons
    margin-top 10px

.g-scope-selection-container .checkbox
    margin-left 25px

    &.disabled
      color #999
